/*
 * This is the main program file for the Instrumented Football Helmet
 *
 * For this project, the PIC18F45K20 is used with the following peripherals:
 *
 *	ADC:
 *    AN0,1,4,5,8-12
 *    using external Vref- and Vref+
 *    Sampling at ~400 Hz
 *
 *  SPI
 *    Used for SD card interface
 *    Use SDO, SDI, and SCK
 *    Need to keep clock rate down at least during card handshaking
 *
 *  USART (RS232-C)
 *    Used for Matchport interface
 * 	  Tx, Rx
 *
 *	Timer0
 */

/*
 * Matchport interfacing
 * 
 * using USART (RS232-C)
 * 9600 bits/sec baud rate
 * Tx, Rx pins
 *
 * Transmit:
 * use interrupts at PIR1bits.TXIF
 * enable these interrupts every time a
 * command/string is ready to be sent, and
 * disable it whenever the command has been sent
 *
 * Receive:
 * use interrupts at PIR1bits.RXIF
 * these interrupts only need to be enabled
 * during startup and then the interrupt will
 * only trigger when a byte has been received
 *
 * have to recognize certain commands/requests
 * sent from the matchport and then send a response
 *
 * Inputs to recognize:
 * Command 1: 0x01 - store the following four values received as thresholds
 *                   1: linear impact concussion alert threshold
 *                   2: linear impact recording threshold
 *                   3: rotational concussion alert threshold
 *                   4: rotational recording threshold
 *                   maybe return a bitwise or of all four imputs or just some sort of receive flag
 * Command 2: 0x02 - send all data stored on SD card to matchport
 * Command 3: 0x03 - reformat SD card
 */

/*
 * SD card interfacing
 *
 * Upon power reset:
 * send CMD0: 40h 00h 00h 00h 00h 95h
 *
 * access considerations:
 *   Possibility 1:
 *     Whenever enough data has been stored in data or 
 *     enough time has been elapsed between impacts, write
 *     that data to the SD card. Then increment the current 
 *     sd write address to immediately after the last piece 
 *     of data written to the memory card.
 *
 *     On a reset, after setting the card to idle the card is 
 *     read until an unused memory location is found.  At this 
 *     point, an EOF is written to this memory location and the
 *     current "write to" address is incremented.
 *
 *     TO DO: see if the SD card can be written to at, say 0x0000
 * 			  and then written to again at 0x0002.  if this is the
 *			  case, this design will work.  Otherwise, something 
 *            else will have to be thought of
 */
/*
 *	Calculation possibilities:
 *		1. Use ADC interrupts to buffer the inputs as 
 * 		   we are calculating the values
 *		2. Keep interrupts off on the ADC and just 
 *		   read each ADC in order, then begin calculations
 *
 *	Issues: might not be able to do any calculations without at least three
 *			inputs (x, y, z) to do resultant vector calculation
 */
#include <p18cxxx.h>
#include <delays.h>
#include <timers.h>
#include <usart.h>
#include <adc.h>
#include <spi.h>
#include <math.h>
//#include "rev2_SD.h"
#include <p18cxxx.h>

#define OK 0x00
#define PRINT_ERR 0x01
#define INIT_ERR 0x02
#define SD_ERR 0x03

#define SD_BLOCK_SIZE 128

#define LNR_THRSH_REC_ADDR 0x00
#define LNR_THRSH_WRN_ADDR 0x01
#define	RTN_THRSH_REC_ADDR 0x02
#define	RTN_THRSH_WRN_ADDR 0x03
#define	BATSTAT_ADDR 0x04
#define SECCNT1 18
#define SECCNT2 217
#define ADC_RATE 10
#define bfrsize 5
#define ADCREADS 5

void low_intr(void);
void high_intr(void);
void writeString (const rom char *, int);
void checkCmd(void);
void SetVariable(char);
void ReadVariable(char);
void WriteEEPROM(int, char);
void wait_on_wr(void);
void writeVar(unsigned int);
int TestAccOne(void);
int TestAccTwo(void);
int TestAccThree(void);
char getAdcChannel(int);
int getVar(int);
void setVar(unsigned int, int);
void printAccel(const rom char*, int);
void AccP(void);
void init(void);
void initAccels(void);
void writeVariable(char [], int);
void writeCharacter(char);
int abs(int);
void calc(void);
void printSDRecords (void);
void initFromSD(void);
void resetSD(void);
void newBlock(unsigned long loc);
void writeSDRec( unsigned char level);
//void testCurSDVals(void);

//SD Functions
char setup_SDSPI( void );
char SDReadBlock( unsigned long );
char SDWriteBlock(unsigned long );
void InitSPI( void );
char InitSD( void );
char SD_WriteCommand(char* cmd);
char SPIRead( void );
void SPIWrite(char data);
void SD_setStart( void );
void SD_SetCurrBlock(unsigned long newblock);
unsigned long SD_GetCurrBlock(void);
unsigned char SD_GetSample( int cnt );
void SD_SetSample( int cnt, unsigned char sample );
void SD_writeCurr( void );
void SD_readCurr( void );
void WriteSamples(char* buff);


